function [xi_AIEM,Pi_AIEM] = AIEM(A,y,T,epsilon)

% Albebraically Initialized Expectation Maximization (AIEM)
% for the Shuffled Linear Regression (SLR) problem (A,y)
% M.C. Tsakiris et al., arXiv, 2018.
% Implementation only for n=3,4. For n>4 please use a general polynomial
% system solver such as Bertini.

% INPUT
% A: m x n coefficient matrix
% y: m x 1 permuted observations
% T: maximum iterations of the hardEM algorithm of Abid et al., 2018
% epsilon: termination precision of the EM algorithm

% OUTPUT
% x: solution to the SLR problem
% Pi: corresponding permutation matching the rows of A to y (applied on A)

% Copyright @ M.C. Tsakiris, Liangzu Peng, L. Kneip, 2018.

% dimensions
[~,n] = size(A);

% compute the real part of the roots of the first n power sum polynomials p_k
if n == 3
    Xi_AI_hat = sympol3Mat(A, y).';   
elseif n == 4
    Xi_AI_hat = sympol4Mat(A, y).';
else
   display("error: for n>4 please use Bertini or some other polynomial system solver");
end

% estimate the most suitable root of the polynomial system
e_AI_hat = vecnorm(sort(y) - sort(A*Xi_AI_hat, 1), 2, 1);
[~, index] = min(e_AI_hat);
xi_AI_hat = squeeze(Xi_AI_hat(:, index));

% refine the root by alternating minimization
[xi_AIEM,Pi_AIEM] = SLR_hardEM(A,y,xi_AI_hat,T,epsilon);

end
